@@ -30,22 +30,22 @@
 /* Recognizing special file names. */
 gboolean nautilus_file_name_matches_hidden_pattern   (const char *name_or_relative_uri);
 gboolean nautilus_file_name_matches_backup_pattern   (const char *name_or_relative_uri);
-gboolean nautilus_file_name_matches_metafile_pattern (const char *name_or_relative_uri);
 
-/* FIXME bugzilla.gnome.org 42424: 
- * This is the same as gnome-libs g_concat_dir_and_file except
- * for handling path == NULL.
- */
-char *   nautilus_make_path                          (const char *path,
-						      const char *name);
 /* These functions all return something something that needs to be
  * freed with g_free, is not NULL, and is guaranteed to exist.
  */
 char *   nautilus_get_user_directory                 (void);
 char *   nautilus_get_desktop_directory              (void);
+char *   nautilus_get_desktop_directory_uri          (void);
+gboolean nautilus_is_desktop_directory_escaped       (char *escaped_dir,
+						      char *escaped_filename);
 char *   nautilus_get_gmc_desktop_directory          (void);
 char *   nautilus_get_pixmap_directory               (void);
 
+/* This function returns something that needs to be freed with g_free,
+ * is not NULL, but is not garaunteed to exist */
+char *   nautilus_get_desktop_directory_uri_no_create (void);
+
 /* A version of gnome's gnome_pixmap_file that works for the nautilus prefix.
  * Otherwise similar to gnome_pixmap_file in that it checks to see if the file
  * exists and returns NULL if it doesn't.
@@ -59,17 +59,6 @@ char *   nautilus_pixmap_file                        (const char *partial_path);
 /* Locate a file in either the uers directory or the datadir. */
 char *   nautilus_get_data_file_path                 (const char *partial_path);
 
-/* Returns the build time stamp the Nautilus binary.
- * This is useful to be able to tell builds apart.
- * A return value of NULL means unknown time stamp.
- */
-char *   nautilus_get_build_time_stamp               (void);
-
-/* Returns the "build message", which provides some information on build
- * context.  May return NULL.
- */
-char *   nautilus_get_build_message                  (void);
-
 /* Return an allocated file name that is guranteed to be unique. */
 char *   nautilus_unique_temporary_file_name         (void);
 char *   nautilus_find_file_in_gnome_path            (char       *file);
