@@ -63,6 +63,7 @@
 
 #define MAX_TEXT_WIDTH_STANDARD 135
 #define MAX_TEXT_WIDTH_TIGHTER 80
+#define MAX_TEXT_WIDTH_BESIDE 90
 
 /* Private part of the NautilusIconCanvasItem structure. */
 struct NautilusIconCanvasItemDetails {
@@ -93,9 +94,12 @@ struct NautilusIconCanvasItemDetails {
 	guint rendered_is_highlighted_for_selection : 1;
 	guint rendered_is_highlighted_for_drop : 1;
 	guint rendered_is_prelit : 1;
+	guint rendered_is_focused : 1;
 	
 	guint is_renaming : 1;
-
+	
+	guint bounds_cached : 1;
+	
 	PangoLayout *editable_text_layout;
 	PangoLayout *additional_text_layout;
 	
@@ -107,6 +111,8 @@ struct NautilusIconCanvasItemDetails {
 	ArtIRect text_rect;
 	ArtIRect emblem_rect;
 
+	ArtIRect bounds_cache;
+	
 	/* Accessibility bits */
 	GailTextUtil *text_util;
 };
@@ -148,12 +154,10 @@ static void     nautilus_icon_canvas_item_init       (NautilusIconCanvasItem
 /* private */
 static void     draw_or_measure_label_text           (NautilusIconCanvasItem        *item,
 						      GdkDrawable                   *drawable,
-						      int                            icon_left,
-						      int                            icon_bottom);
+						      ArtIRect                       icon_rect);
 static void     draw_label_text                      (NautilusIconCanvasItem        *item,
 						      GdkDrawable                   *drawable,
-						      int                            icon_left,
-						      int                            icon_bottom);
+						      ArtIRect                       icon_rect);
 static void     measure_label_text                   (NautilusIconCanvasItem        *item);
 static void     get_icon_canvas_rectangle            (NautilusIconCanvasItem        *item,
 						      ArtIRect                      *rect);
@@ -272,10 +276,17 @@ pixbuf_is_acceptable (GdkPixbuf *pixbuf)
 		&& gdk_pixbuf_get_bits_per_sample (pixbuf) == 8;
 }
 
+static void
+nautilus_icon_canvas_item_invalidate_bounds_cache (NautilusIconCanvasItem *item)
+{
+	item->details->bounds_cached = FALSE;
+}
+
 /* invalidate the text width and height cached in the item details. */
 void
 nautilus_icon_canvas_item_invalidate_label_size (NautilusIconCanvasItem *item)
 {
+	nautilus_icon_canvas_item_invalidate_bounds_cache (item);
 	item->details->text_width = -1;
 	item->details->text_height = -1;
 	if (item->details->editable_text_layout != NULL) {
@@ -452,6 +463,7 @@ nautilus_icon_canvas_item_set_image (NautilusIconCanvasItem *item,
 
 	details->pixbuf = image;
 			
+	nautilus_icon_canvas_item_invalidate_bounds_cache (item);
 	eel_canvas_item_request_update (EEL_CANVAS_ITEM (item));	
 }
 
@@ -481,6 +493,8 @@ nautilus_icon_canvas_item_set_emblems (NautilusIconCanvasItem *item,
 	eel_gdk_pixbuf_list_ref (emblem_pixbufs);
 	eel_gdk_pixbuf_list_free (item->details->emblem_pixbufs);
 	item->details->emblem_pixbufs = g_list_copy (emblem_pixbufs);
+
+	nautilus_icon_canvas_item_invalidate_bounds_cache (item);
 	eel_canvas_item_request_update (EEL_CANVAS_ITEM (item));
 }
 
@@ -495,6 +509,8 @@ nautilus_icon_canvas_item_set_attach_points (NautilusIconCanvasItem *item,
 		item->details->attach_points = g_new (NautilusEmblemAttachPoints, 1);
 		*item->details->attach_points = *attach_points;
 	}
+	
+	nautilus_icon_canvas_item_invalidate_bounds_cache (item);
 }
 
 void
@@ -503,6 +519,7 @@ nautilus_icon_canvas_item_set_embedded_text_rect (NautilusIconCanvasItem       *
 {
 	item->details->embedded_text_rect = *text_rect;
 
+	nautilus_icon_canvas_item_invalidate_bounds_cache (item);
 	eel_canvas_item_request_update (EEL_CANVAS_ITEM (item));
 }
 
@@ -569,22 +586,35 @@ recompute_bounding_box (NautilusIconCanvasItem *icon_item,
 }
 
 static ArtIRect
-compute_text_rectangle (NautilusIconCanvasItem *item,
-			ArtIRect icon_rectangle)
+compute_text_rectangle (const NautilusIconCanvasItem *item,
+			ArtIRect icon_rectangle,
+			gboolean canvas_coords)
 {
 	ArtIRect text_rectangle;
 	double pixels_per_unit;
 	double text_width, text_height;
 
 	pixels_per_unit = EEL_CANVAS_ITEM (item)->canvas->pixels_per_unit;
-	text_width = item->details->text_width / pixels_per_unit;
-	text_height = item->details->text_height / pixels_per_unit;
-
-	/* Compute text rectangle. */
-	text_rectangle.x0 = (icon_rectangle.x0 + icon_rectangle.x1) / 2 - text_width / 2;
-	text_rectangle.y0 = icon_rectangle.y1;
-	text_rectangle.x1 = text_rectangle.x0 + text_width;
-	text_rectangle.y1 = text_rectangle.y0 + text_height + LABEL_OFFSET / pixels_per_unit;
+	if (canvas_coords) {
+		text_width = item->details->text_width;
+		text_height = item->details->text_height;
+	} else {
+		text_width = item->details->text_width / pixels_per_unit;
+		text_height = item->details->text_height / pixels_per_unit;
+	}
+	
+	if (NAUTILUS_ICON_CONTAINER (EEL_CANVAS_ITEM (item)->canvas)->details->label_position == NAUTILUS_ICON_LABEL_POSITION_BESIDE) {
+                text_rectangle.x0 = icon_rectangle.x1;
+                text_rectangle.x1 = text_rectangle.x0 + text_width;
+                text_rectangle.y0 = icon_rectangle.y0;
+                text_rectangle.y1 = text_rectangle.y0 + text_height + LABEL_OFFSET / pixels_per_unit;
+	} else {
+                text_rectangle.x0 = (icon_rectangle.x0 + icon_rectangle.x1) / 2
+			- text_width / 2;
+                text_rectangle.y0 = icon_rectangle.y1;
+                text_rectangle.x1 = text_rectangle.x0 + text_width;
+                text_rectangle.y1 = text_rectangle.y0 + text_height + LABEL_OFFSET / pixels_per_unit;
+        }
 
 	return text_rectangle;
 }
@@ -627,7 +657,7 @@ nautilus_icon_canvas_item_update_bounds (NautilusIconCanvasItem *item,
 	
 	/* Update canvas and text rect cache */
 	get_icon_canvas_rectangle (item, &item->details->canvas_rect);
-	item->details->text_rect = compute_text_rectangle (item, item->details->canvas_rect);
+	item->details->text_rect = compute_text_rectangle (item, item->details->canvas_rect, FALSE);
 	
 	/* Update emblem rect cache */
 	item->details->emblem_rect.x0 = 0;
@@ -642,7 +672,7 @@ nautilus_icon_canvas_item_update_bounds (NautilusIconCanvasItem *item,
 	/* queue a redraw. */
 	eel_canvas_request_redraw (canvas_item->canvas,
 				   before.x0, before.y0,
-				   before.x1, before.y1);
+				   before.x1 + 1, before.y1 + 1);
 }
 
 /* Update handler for the icon canvas item. */
@@ -761,8 +791,7 @@ draw_frame (NautilusIconCanvasItem *item,
 static void
 draw_or_measure_label_text (NautilusIconCanvasItem *item,
 			    GdkDrawable *drawable,
-			    int icon_left,
-			    int icon_bottom)
+			    ArtIRect icon_rect)
 {
 	NautilusIconCanvasItemDetails *details;
 	NautilusIconContainer *container;
@@ -774,12 +803,14 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 	GdkColor *label_color;
 	int icon_width;
 	gboolean have_editable, have_additional, needs_highlight, needs_frame;
-	int max_text_width, box_left;
+	int max_text_width;
+	int x;
 	GdkGC *gc;
+	ArtIRect text_rect;
 	
 	icon_width = 0;
 	gc = NULL;
-	
+
 	details = item->details;
 	needs_highlight = details->is_highlighted_for_selection || details->is_highlighted_for_drop;
 
@@ -848,7 +879,12 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 	}
 
 	details->text_width = MAX (editable_width, additional_width);
-	details->text_height = editable_height + LABEL_LINE_SPACING + additional_height;
+
+	if (have_additional) {
+		details->text_height = editable_height + LABEL_LINE_SPACING + additional_height;
+	} else {
+		details->text_height = editable_height;
+	}
 
 	if (ANTIALIAS_SELECTION_RECTANGLE) {
 		/* add some extra space for highlighting even when we don't highlight so things won't move */
@@ -871,6 +907,8 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 
 		return;
 	}
+
+	text_rect = compute_text_rectangle (item, icon_rect, TRUE);
 	
 	/* if the icon is highlighted, do some set-up */
 	if (needs_highlight && !details->is_renaming &&
@@ -878,12 +916,19 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 		draw_frame (item,
 			    drawable,
 			    GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (container)) ? container->details->highlight_color_rgba : container->details->active_color_rgba,
-			    icon_left + (icon_width - details->text_width) / 2,
-			    icon_bottom,
-			    details->text_width, 
-			    details->text_height);
+			    text_rect.x0,
+			    text_rect.y0,
+			    text_rect.x1 - text_rect.x0,
+			    text_rect.y1 - text_rect.y0);
 	}
 
+		
+	if (container->details->label_position == NAUTILUS_ICON_LABEL_POSITION_BESIDE) {
+		x = text_rect.x0 + 2;
+	} else {
+		x = text_rect.x0 + ((text_rect.x1 - text_rect.x0) - max_text_width) / 2;
+	}
+	
 	if (have_editable) {
 		gtk_widget_style_get (GTK_WIDGET (container),
 				      "frame_text", &needs_frame,
@@ -892,21 +937,21 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 			draw_frame (item, 
 				    drawable,
 				    eel_gdk_color_to_rgb (&GTK_WIDGET (container)->style->base[GTK_STATE_NORMAL]),
-				    icon_left + (icon_width - details->text_width) / 2,
-				    icon_bottom,
-				    details->text_width, 
-				    details->text_height);	    
+				    text_rect.x0,
+				    text_rect.y0,
+				    text_rect.x1 - text_rect.x0,
+				    text_rect.y1 - text_rect.y0);
 		}
 		
 		gc = nautilus_icon_container_get_label_color_and_gc
 			(NAUTILUS_ICON_CONTAINER (canvas_item->canvas),
 			 &label_color, TRUE, needs_highlight);
-		
+
 		draw_label_layout (item, drawable,
 				   editable_layout, needs_highlight,
 				   label_color,
-				   icon_left + (icon_width - max_text_width) / 2,
-				   icon_bottom, gc);
+				   x,
+				   text_rect.y0, gc);
 	}
 
 	if (have_additional) {
@@ -917,12 +962,10 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 		draw_label_layout (item, drawable,
 				   additional_layout, needs_highlight,
 				   label_color,
-				   icon_left + (icon_width - max_text_width) / 2,
-				   icon_bottom + editable_height + LABEL_LINE_SPACING, gc);
+				   x,
+				   text_rect.y0 + editable_height + LABEL_LINE_SPACING, gc);
 	}
 
-	box_left = icon_left + (icon_width - details->text_width) / 2;
-
 	if (item->details->is_highlighted_as_keyboard_focus) {
 		gtk_paint_focus (GTK_WIDGET (EEL_CANVAS_ITEM (item)->canvas)->style,
 				 drawable,
@@ -930,10 +973,10 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 				 NULL,
 				 GTK_WIDGET (EEL_CANVAS_ITEM (item)->canvas),
 				 "icon-container",
-				 box_left,
-				 icon_bottom,
-				 details->text_width,
-				 details->text_height);
+				 text_rect.x0,
+				 text_rect.y0,
+				 text_rect.x1 - text_rect.x0,
+				 text_rect.y1 - text_rect.y0);
 	}
 
 	if (editable_layout) {
@@ -948,6 +991,8 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 static void
 measure_label_text (NautilusIconCanvasItem *item)
 {
+	ArtIRect rect = {0, };
+	
 	/* check to see if the cached values are still valid; if so, there's
 	 * no work necessary
 	 */
@@ -956,14 +1001,14 @@ measure_label_text (NautilusIconCanvasItem *item)
 		return;
 	}
 	
-	draw_or_measure_label_text (item, NULL, 0, 0);
+	draw_or_measure_label_text (item, NULL, rect);
 }
 
 static void
 draw_label_text (NautilusIconCanvasItem *item, GdkDrawable *drawable,
-		 int icon_left, int icon_bottom)
+		 ArtIRect icon_rect)
 {
-	draw_or_measure_label_text (item, drawable, icon_left, icon_bottom + LABEL_OFFSET);
+	draw_or_measure_label_text (item, drawable, icon_rect);
 }
 
 static void
@@ -1225,13 +1270,20 @@ real_map_pixbuf (NautilusIconCanvasItem *icon_item)
 			g_free (audio_filename);
 		}
 	}
-	
+
 	if (icon_item->details->is_highlighted_for_selection
 	    || icon_item->details->is_highlighted_for_drop) {
+		guint color;
+
 		old_pixbuf = temp_pixbuf;
-		temp_pixbuf = eel_create_darkened_pixbuf (temp_pixbuf,
-							  0.8 * 255,
-							  0.8 * 255);
+
+		color =  GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (canvas)) ? NAUTILUS_ICON_CONTAINER (canvas)->details->highlight_color_rgba : NAUTILUS_ICON_CONTAINER (canvas)->details->active_color_rgba;
+
+		temp_pixbuf = eel_create_colorized_pixbuf (temp_pixbuf,
+							   EEL_RGBA_COLOR_GET_R (color),
+							   EEL_RGBA_COLOR_GET_G(color),
+							   EEL_RGBA_COLOR_GET_B (color));
+							   
 		g_object_unref (old_pixbuf);
 	} 
 
@@ -1245,7 +1297,8 @@ map_pixbuf (NautilusIconCanvasItem *icon_item)
 	      && icon_item->details->rendered_is_active == icon_item->details->is_active
 	      && icon_item->details->rendered_is_prelit == icon_item->details->is_prelit
 	      && icon_item->details->rendered_is_highlighted_for_selection == icon_item->details->is_highlighted_for_selection
-	      && icon_item->details->rendered_is_highlighted_for_drop == icon_item->details->is_highlighted_for_drop)) {
+	      && icon_item->details->rendered_is_highlighted_for_drop == icon_item->details->is_highlighted_for_drop
+	      && (icon_item->details->is_highlighted_for_selection && icon_item->details->rendered_is_focused == GTK_WIDGET_HAS_FOCUS (EEL_CANVAS_ITEM (icon_item)->canvas)))) {
 		if (icon_item->details->rendered_pixbuf != NULL) {
 			g_object_unref (icon_item->details->rendered_pixbuf);
 		}
@@ -1254,6 +1307,7 @@ map_pixbuf (NautilusIconCanvasItem *icon_item)
 		icon_item->details->rendered_is_prelit = icon_item->details->is_prelit;
 		icon_item->details->rendered_is_highlighted_for_selection = icon_item->details->is_highlighted_for_selection;
 		icon_item->details->rendered_is_highlighted_for_drop = icon_item->details->is_highlighted_for_drop;
+		icon_item->details->rendered_is_focused = GTK_WIDGET_HAS_FOCUS (EEL_CANVAS_ITEM (icon_item)->canvas);
 	}
 
 	g_object_ref (icon_item->details->rendered_pixbuf);
@@ -1302,6 +1356,7 @@ nautilus_icon_canvas_item_draw (EelCanvasItem *item, GdkDrawable *drawable,
 	ArtIRect icon_rect, emblem_rect;
 	EmblemLayout emblem_layout;
 	GdkPixbuf *emblem_pixbuf, *temp_pixbuf;
+	GdkRectangle draw_rect, pixbuf_rect;
 			
 	icon_item = NAUTILUS_ICON_CANVAS_ITEM (item);
 	details = icon_item->details;
@@ -1311,12 +1366,27 @@ nautilus_icon_canvas_item_draw (EelCanvasItem *item, GdkDrawable *drawable,
 		return;
 	}
 
-	/* Compute icon rectangle in drawable coordinates. */
 	icon_rect = icon_item->details->canvas_rect;
-
+	
 	/* if the pre-lit or selection flag is set, make a pre-lit or darkened pixbuf and draw that instead */
 	temp_pixbuf = map_pixbuf (icon_item);
-	draw_pixbuf (temp_pixbuf, drawable, icon_rect.x0, icon_rect.y0);
+	pixbuf_rect.x = icon_rect.x0;
+	pixbuf_rect.y = icon_rect.y0;
+	pixbuf_rect.width = gdk_pixbuf_get_width (temp_pixbuf);
+	pixbuf_rect.height = gdk_pixbuf_get_height (temp_pixbuf);
+	if (gdk_rectangle_intersect (&(expose->area), &pixbuf_rect, &draw_rect)) {
+		gdk_draw_pixbuf (drawable,
+				 NULL,
+				 temp_pixbuf,
+				 draw_rect.x - pixbuf_rect.x,
+				 draw_rect.y - pixbuf_rect.y,
+				 draw_rect.x,
+				 draw_rect.y,
+				 draw_rect.width,
+				 draw_rect.height,
+				 GDK_RGB_DITHER_NORMAL,
+				 0,0);
+	}
 	g_object_unref (temp_pixbuf);
 
 	draw_embedded_text (icon_item, drawable,  icon_rect.x0, icon_rect.y0);
@@ -1331,7 +1401,7 @@ nautilus_icon_canvas_item_draw (EelCanvasItem *item, GdkDrawable *drawable,
 	draw_stretch_handles (icon_item, drawable, &icon_rect);
 	
 	/* Draw the label text. */
-	draw_label_text (icon_item, drawable, icon_rect.x0, icon_rect.y1);
+	draw_label_text (icon_item, drawable, icon_rect);
 }
 
 static PangoLayout *
@@ -1352,7 +1422,13 @@ create_label_layout (NautilusIconCanvasItem *item,
 
 	pango_layout_set_text (layout, text, -1);
 	pango_layout_set_width (layout, floor (nautilus_icon_canvas_item_get_max_text_width (item)) * PANGO_SCALE);
-	pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+			
+	if (container->details->label_position == NAUTILUS_ICON_LABEL_POSITION_BESIDE) {
+		pango_layout_set_alignment (layout, PANGO_ALIGN_LEFT);
+	} else {
+		pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+	}
+
 	pango_layout_set_spacing (layout, LABEL_LINE_SPACING);
 	pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
 
@@ -1669,7 +1745,7 @@ nautilus_icon_canvas_item_bounds (EelCanvasItem *item,
 	double pixels_per_unit;
 	EmblemLayout emblem_layout;
 	GdkPixbuf *emblem_pixbuf;
-	
+
 	g_assert (x1 != NULL);
 	g_assert (y1 != NULL);
 	g_assert (x2 != NULL);
@@ -1678,35 +1754,42 @@ nautilus_icon_canvas_item_bounds (EelCanvasItem *item,
 	icon_item = NAUTILUS_ICON_CANVAS_ITEM (item);
 	details = icon_item->details;
 
-	measure_label_text (icon_item);
+	if (details->bounds_cached) {
+		total_rect = details->bounds_cache;
+	} else {	
+		measure_label_text (icon_item);
+		
+		/* Compute icon rectangle. */
+		icon_rect.x0 = 0;
+		icon_rect.y0 = 0;
+		if (details->pixbuf == NULL) {
+			icon_rect.x1 = icon_rect.x0;
+			icon_rect.y1 = icon_rect.y0;
+		} else {
+			pixels_per_unit = item->canvas->pixels_per_unit;
+			icon_rect.x1 = icon_rect.x0 + gdk_pixbuf_get_width (details->pixbuf) / pixels_per_unit;
+			icon_rect.y1 = icon_rect.y0 + gdk_pixbuf_get_height (details->pixbuf) / pixels_per_unit;
+		}
+		
+		/* Compute text rectangle. */
+		text_rect = compute_text_rectangle (icon_item, icon_rect, FALSE);
+		
+		/* Compute total rectangle, adding in emblem rectangles. */
+		art_irect_union (&total_rect, &icon_rect, &text_rect);
+		emblem_layout_reset (&emblem_layout, icon_item, icon_rect);
+		while (emblem_layout_next (&emblem_layout, &emblem_pixbuf, &emblem_rect)) {
+			art_irect_union (&total_rect, &total_rect, &emblem_rect);
+		}
 
-	/* Compute icon rectangle. */
-	icon_rect.x0 = details->x;
-	icon_rect.y0 = details->y;
-	if (details->pixbuf == NULL) {
-		icon_rect.x1 = icon_rect.x0;
-		icon_rect.y1 = icon_rect.y0;
-	} else {
-		pixels_per_unit = item->canvas->pixels_per_unit;
-		icon_rect.x1 = icon_rect.x0 + gdk_pixbuf_get_width (details->pixbuf) / pixels_per_unit;
-		icon_rect.y1 = icon_rect.y0 + gdk_pixbuf_get_height (details->pixbuf) / pixels_per_unit;
-	}
-	
-	/* Compute text rectangle. */
-	text_rect = compute_text_rectangle (icon_item, icon_rect);
-
-	/* Compute total rectangle, adding in emblem rectangles. */
-	art_irect_union (&total_rect, &icon_rect, &text_rect);
-	emblem_layout_reset (&emblem_layout, icon_item, icon_rect);
-	while (emblem_layout_next (&emblem_layout, &emblem_pixbuf, &emblem_rect)) {
-		art_irect_union (&total_rect, &total_rect, &emblem_rect);
+		details->bounds_cache = total_rect;
+		details->bounds_cached = TRUE;
 	}
         
 	/* Return the result. */
-	*x1 = floor (total_rect.x0);
-	*y1 = floor (total_rect.y0);
-	*x2 = ceil (total_rect.x1) + 1;
-	*y2 = ceil (total_rect.y1) + 1;
+	*x1 = (int)details->x + total_rect.x0;
+	*y1 = (int)details->y + total_rect.y0;
+	*x2 = (int)details->x + total_rect.x1 + 1;
+	*y2 = (int)details->y + total_rect.y1 + 1;
 }
 
 /* Get the rectangle of the icon only, in world coordinates. */
@@ -1738,6 +1821,45 @@ nautilus_icon_canvas_item_get_icon_rectangle (const NautilusIconCanvasItem *item
 	return rectangle;
 }
 
+ArtDRect
+nautilus_icon_canvas_item_get_text_rectangle (const NautilusIconCanvasItem *item)
+{
+	/* FIXME */
+	ArtIRect icon_rectangle;
+	ArtIRect text_rectangle;
+	ArtDRect ret;
+	double pixels_per_unit;
+	GdkPixbuf *pixbuf;
+	
+	g_return_val_if_fail (NAUTILUS_IS_ICON_CANVAS_ITEM (item), eel_art_drect_empty);
+
+	icon_rectangle.x0 = item->details->x;
+	icon_rectangle.y0 = item->details->y;
+	
+	pixbuf = item->details->pixbuf;
+	
+	pixels_per_unit = EEL_CANVAS_ITEM (item)->canvas->pixels_per_unit;
+	icon_rectangle.x1 = icon_rectangle.x0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_width (pixbuf)) / pixels_per_unit;
+	icon_rectangle.y1 = icon_rectangle.y0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_height (pixbuf)) / pixels_per_unit;
+
+	text_rectangle = compute_text_rectangle (item, icon_rectangle, FALSE);
+ 
+	ret.x0 = text_rectangle.x0;
+	ret.y0 = text_rectangle.y0;
+	ret.x1 = text_rectangle.x1;
+	ret.y1 = text_rectangle.y1;
+
+        eel_canvas_item_i2w (EEL_CANVAS_ITEM (item),
+                             &ret.x0,
+                             &ret.y0);
+        eel_canvas_item_i2w (EEL_CANVAS_ITEM (item),
+                             &ret.x1,
+                             &ret.y1);
+ 
+        return ret;
+}
+
+
 /* Get the rectangle of the icon only, in canvas coordinates. */
 static void
 get_icon_canvas_rectangle (NautilusIconCanvasItem *item,
@@ -1875,7 +1997,14 @@ nautilus_icon_canvas_item_get_max_text_width (NautilusIconCanvasItem *item)
 	if (nautilus_icon_container_is_tighter_layout (NAUTILUS_ICON_CONTAINER (canvas_item->canvas))) {
 		return MAX_TEXT_WIDTH_TIGHTER * canvas_item->canvas->pixels_per_unit;
 	} else {
-		return MAX_TEXT_WIDTH_STANDARD * canvas_item->canvas->pixels_per_unit;
+				
+                if (NAUTILUS_ICON_CONTAINER (canvas_item->canvas)->details->label_position == NAUTILUS_ICON_LABEL_POSITION_BESIDE) {
+			return MAX_TEXT_WIDTH_BESIDE * canvas_item->canvas->pixels_per_unit;
+                } else {
+			return MAX_TEXT_WIDTH_STANDARD * canvas_item->canvas->pixels_per_unit;
+                }
+
+
 	}
 
 }
