@@ -1909,7 +1909,7 @@ nautilus_window_display_error (NautilusWindow *window, const char *error_msg)
 	GtkWidget *dialog;
 	
 	dialog = gtk_message_dialog_new (GTK_WINDOW (window), 0, GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE, error_msg, NULL);
+					 GTK_BUTTONS_OK, error_msg, NULL);
 	gtk_widget_show (dialog);
 }
 
